from math import *
from util import format_number

class Vector3(object):
    
    __slots__ = ('_x', '_y', '_z')
       
    def __init__(self, *args):
        """Creates a Vector3 from 3 numeric values or a list-like object
        containing at least 3 values.
        
        """
        if len(args) == 3:
            self._x = float(args[0])
            self._y = float(args[1])
            self._z = float(args[2])
            return
        
        if not args:
            self._x = self._y = self.z = 0.
        elif len(args) == 1:
            values = args[0]
            self._x = float(values[0])
            self._y = float(values[1])
            self._z = float(values[2])
        else:
            raise ValueError("Vector3.__init__ takes 0, 1 or 3 values")
       
    @classmethod
    def from_3_floats(cls, x, y, z):
        v = cls.__new__(cls, object)
        v._x = x
        v._y = y
        v._z = z
        return v

    @classmethod
    def from_iterable(cls, iterable):
        """Creates a Vector3 from an iterable containing at least 3 values."""
        it = iter(iterable)
        v = cls.__new__(cls, object)
        v._x = float(it.next())
        v._y = float(it.next())
        v._z = float(it.next())
        return v
    
    def copy():
        """Returns a copy of this vector."""
        return Vector3.from_3_floats(v3._x, v3._y, v3._z)

          
    def get_x(self):
        return self._x
    def set_x(self, x):
        self._x = float(x)
    x = property(get_x, set_x, None, "x component.")
    
    def get_y(self):
        return self._y
    def set_y(self, y):
        self._y = float(y)
    y = property(get_y, set_y, None, "y component.")
    
    def get_z(self):
        return self._z
    def set_z(self, z):
        self._z = float(z)
    z = property(get_z, set_z, None, "z component.")
        
    def _get_length(self):
        return sqrt(self._x*self._x+self._y*self._y+self._z*self._z)
    def _set_length(self, length):
        try:
            l = length / sqrt(self._x*self._x+self._y*self._y+self._z*self._z)
        except ZeroDivisionError:
            self._x = 0.
            self._y = 0.
            self._z = 0.
            
        self._x *= l
        self._y *= l
        self._z *= l
        
    length = property(_get_length, _set_length, None, "Length of the vector")
    
    def unit(self):
        l = sqrt(self._x*self._x+self._y*self._y+self._z*self._z)        
        return Vector3.from_3_floats(self._x/l, self._y/l, self._z/l)            
                
        
    def set(self, x, y, z):
        """Sets the components of this vector.
        x -- x component
        y -- y component
        z -- z component"""
        v._x = float(x)
        v._y = float(y)
        v._z = float(z)
        return self
        
            
    def __str__(self):
        
        return "(%s, %s, %s)" % (format_number(self._x), format_number(self._y), format_number(self._z))
        
        
    def __repr__(self):
        
        return "Vector3(%s, %s, %s)" % (self._x, self._y, self._z)        
        
        
    def __len__(self):
        return 3
    
    def __iter__(self):
                
        yield self._x
        yield self._y
        yield self._z     
        
        
    def __getitem__(self, index):
       
        try:
            return getattr(self, self.__slots__[index])            
        except IndexError:
            raise IndexError, "There are 3 values in this object, index should be 0, 1 or 2!"        
        
        
    def __setitem__(self, index, value):
        
        try:
            setattr(self, self.__slots__[index], value)
        except IndexError:
            raise IndexError, "There are 3 values in this object, index should be 0, 1 or 2!"


    def __add__(self, rhs):
        """Returns the result of adding a vector (or collection of 3 numbers) from this vector."""
        
        return Vector3.from_3_floats(self._x+rhs[0], self._y+rhs[1], self._z+rhs[2])
        
        
    def __iadd__(self, rhs):
        """Adds another vector (or a collection of 3 numbers) to this vector."""
        
        self._x += rhs[0]
        self._y += rhs[1]
        self._z += rhs[2]
        return self
        
        
    def __sub__(self, rhs):
        """Returns the result of subtracting a vector (or collection of 3 numbers) from this vector."""
        
        return Vector3.from_3_floats(self._x-rhs[0], self._y-rhs[1], self._z-rhs[2])
        
        
    def _isub__(self, rhs):
        """Subtracts another vector (or a collection of 3 numbers) from this vector."""
        
        self._x -= rhs[0]
        self._y -= rhs[1]
        self._z -= rhs[2]
        return self
        
        
    def __mul__(self, rhs):
        """Return the result of multiplying this vector by another vector, or a scalar (single number)."""
        
        if hasattr(rhs, "__getitem__"):
            return Vector3.from_3_floats(self._x*rhs[0], self._y*rhs[1], self._z*rhs[2])
        else:            
            return Vector3.from_3_floats(self._x*rhs, self._y*rhs, self._z*rhs)
            
        
    def __imul__(self, rhs):
        """Multiply this vector by another vector, or a scalar (single number).""" 
        
        if hasattr(rhs, "__getitem__"):
            self._x *= rhs[0]
            self._y *= rhs[1]
            self._z *= rhs[2]
        else:
            self._x *= rhs
            self._y *= rhs
            self._z *= rhs 
        return self
        
        
    def __div__(self, rhs):        
        """Return the result of dividing this vector by another vector, or a scalar (single number)."""
        
        if hasattr(rhs, "__getitem__"):
            return Vector3.from_3_floats(self._x/rhs[0], self._y/rhs[1], self._z/rhs[2])
        else:
            return Vector3.from_3_floats(self._x/rhs, self._y/rhs, self._z/rhs)
            
            
    def __idiv__(self, rhs):
        """Divide this vector by another vector, or a scalar (single number)."""
        
        if hasattr(rhs, "__getitem__"):
            self._x /= rhs[0]
            self._y /= rhs[1]
            self._z /= rhs[2]
        else:
            self._x /= rhs
            self._y /= rhs
            self._z /= rhs
        return self
            
            
    def __neg__(self):
        """Returns the negation of this vector (a vector pointing in the opposite direction.
        eg v1 = Vector(1,2,3)
        print -v1
        >>> (-1,-2,-3)
        
        """
        return Vector3.from_3_floats(-self._x, -self._y, -self._z)
    
    
    def __nonzero__(self):
        
        return self._x != 0. and self._y != 0. and self._z != 0.
    
    
    def __call__(self, keys):
        """Returns a tuple of the values in a vector
        
        keys -- An iterable containing the keys (x, y or z)
        eg v = Vector3(1.0, 2.0, 3.0)
        v('zyx') -> (3.0, 2.0, 1.0)
        
        """
        return tuple( getattr(self, "_"+key) for key in keys )
             
             
    def as_tuple(self):
        """Returns a tuple of the x, y, z components. A little quicker than iter(vector)."""
        
        return (self._x, self._y, self._z)
        
        
    def scale(self, scale):
        """Scales the vector by onther vector or a scalar. Same as the *= operator."""
        if hasattr(scale, "__getitem__"):
            self._x *= scale[0]
            self._y *= scale[1]
            self._z *= scale[2]
        else:
            self._x *= scale
            self._y *= scale
            self._z *= scale  
        return self
        
        
    def get_length(self):
        """Calculates the length of the vector."""
        
        return sqrt(self._x*self._x + self._y*self._y + self._z*self._z)
    get_magnitude = get_length
        
    def set_length(self, new_length):
        """Sets the length of the vector. (Normalises it then scales it)
        
        new_length -- The new length of the vector.
        
        """        
        length = self.get_length()
        self._x /= length
        self._y /= length
        self._z /= length
        self._x *= new_length
        self._y *= new_length
        self._z *= new_length
        return self
        
        
    def get_distance_to(self, p):
        """Returns the distance of this vector to a point.
        
        p -- A position as a vector, or collection of 3 values.
        
        """
        
        return sqrt( (self._x - p[0])**2 + (self._y - p[1])**2 + (self._z - p[2])**2 )    
        
        
    def get_distance_squared(self, p):
        """Returns the squared distance of this vector to a point.
        
        p -- A position as a vector, or collection of 3 values.
        
        """
        
        return (self._x - p[0])**2 + (self._y - p[1])**2 + (self._z - p[2])**2
        
        
    def normalise(self):
        """Scales the vector to be length 1."""
        
        length = self.get_length()
        self._x /= length
        self._y /= length
        self._z /= length
        return self        
    normalize = normalise
        
    def get_normalised(self):
        
        length = self.get_length()
        return Vector3.from_3_floats(self._x/length, self._y/length, self._z/length)
    get_normalized = get_normalised
    
    
    def in_sphere(self, sphere):
        """Returns true if this vector (treated as a position) is contained in the given sphere."""
        
        return distance3d(sphere.position, self) <= sphere.radius
     

def distance3d_squared(p1, p2):
    
    return (p2[0]-p1[0])**2 + (p2[1]-p1[1])**2 + (p2[2]-p1[2])**2


def distance3d(p1, p2):
    
    return sqrt( (p2[0]-p1[0])**2 + (p2[1]-p1[1])**2 + (p2[2]-p1[2])**2 )

def centre_point3d(points):
    
    return sum( Vector3(p) for p in points ) / len(points)


if __name__ == "__main__":
    
    v1 = Vector3(2.2323, 3.43242, 1.)
    print v1    
    v2= Vector3(1.2, 5, 10)
    print v2
    v1 += v2
    print v1.get_length()
    print repr(v1)
    print v1[1]
    
    p1 = Vector3(1,2,3)
    print p1
    print repr(p1)    
    
    for v in p1:
        print v
        
    #print p1[6]
    
    ptest = Vector3( [1,2,3] )
    print ptest
    
    z = Vector3()
    print z
        
    file("test.txt", "w").write( "\n".join(str(float(n)) for n in range(20)) )
    f = file("test.txt")
    v1 = Vector3.from_iterable( f )
    v2 = Vector3.from_iterable( f )
    v3 = Vector3.from_iterable( f ) 
    print v1, v2, v3
    
    print "--"
    print v1
    print v1 + (10,20,30)
    
    print v1('xz')
    
    print -v1

    #print tuple(ptest)
    #p1.set( (4, 5, 6) )
    #print p1
    
    print Vector3(10,10,30)+v1
    